﻿// Copyright(c) 2015 DatumTron Corp, All Rights Reserved
// Author: Ashraf Azmi
using System;
using System.Data;
using DatumTron.Datum.Abstract;
using DatumTron.Datum.Atum;

namespace DatumTron.Apps
{
    class Program
    {
        /// <summary>
        /// Loads a Datum Universe created from Microsoft's Northwind SQL database.
        /// Also shows how to import, save, and load your own database
        /// </summary>
        /// <param name="args">not used</param>
        static void Main(string[] args)
        {
            katum root;

            const bool import = true;
            if (import)
            {
                const string sqlConnectionString =
                    @"Data Source = (localdb)\MSSQLLocalDB; Initial Catalog = Northwind; Integrated Security = True; Connect Timeout = 30; Encrypt = False; TrustServerCertificate = False; ApplicationIntent = ReadWrite; MultiSubnetFailover = False";
                root = ImportNorthwind(new atum(),"root", sqlConnectionString);

                Console.WriteLine("=== After import ===");
                PrintSomeStuff(root);
            }

            const bool save = true;
            if (save)
            {
                const string datumFileName = @"..\..\northwind.datum";
                katum.save(root, datumFileName);
            }

            const bool load = true;
            if(load)
            {
                // Load from a datum file
                const string datumFileName = @"..\..\northwind.datum";
                root = katum.load(new atum(), datumFileName);

                Console.WriteLine("=== After load ===");
                PrintSomeStuff(root);
            }
            // -----------------------------------
            Console.WriteLine("\n\nPress any key to exit . . .");
            Console.ReadKey(); 
        }

        /// <summary>
        /// Imports the Northwind database into a Datum Universe
        /// Change the connection string and the DatabaseScheme to import a different database
        /// </summary>
        /// <param name="rootName">the katum root name</param>
        /// <param name="NorthwindConnectionString">the connection string for Northwind</param>
        /// <returns>the root katum</returns>
        static katum ImportNorthwind(IAtum atum, string rootName, string NorthwindConnectionString)
        {
            Console.WriteLine("\n========== Import Started @ {0} ============", DateTime.Now);

            var NorthwindScheme = new DatabaseScheme(new[]
            {
                new TableDescription("Categories", "Category", new [] {"CategoryID"}), 
                new TableDescription("CustomerDemographics", "CustomerDemographic", new [] { "CustomerID", "CustomerTypeID" }), 
                new TableDescription("Customers", "Customer", new [] {"CustomerID"}), 
                new TableDescription("Employees", "Employee", new [] { "EmployeeID" }, new [] { new ForeignKey("ReportsTo", "Employees") }), 
                new TableDescription("Order Details", "Order Detail", new [] { "OrderID", "ProductID" }, new [] { new ForeignKey("OrderID", "Orders"), new ForeignKey("ProductID", "Products")}), 
                new TableDescription("Orders", "Order", new [] { "OrderID" }, new [] { new ForeignKey("CustomerID", "Customers"), new ForeignKey("EmployeeID", "Employees"), new ForeignKey("ShipVia", "Shippers") }),                    
                new TableDescription("Products", "Product", new [] { "ProductID" }, new [] { new ForeignKey("SupplierID", "Suppliers"), new ForeignKey("CategoryID", "Categories") }), 
                new TableDescription("Region", "Region", new [] {"RegionID"}), 
                new TableDescription("Shippers", "Shipper", new [] {"ShipperID"}), 
                new TableDescription("Suppliers", "Supplier", new [] {"SupplierID"}), 
                new TableDescription("Territories", "Territory", new [] { "TerritoryID" }, new [] { new ForeignKey("RegionID", "Region") }), 
            });
            
            // -------------------------------------------------------------------------------------------
            katum root = Importer.ImportDatabase(atum, rootName, NorthwindScheme, NorthwindConnectionString);
            // -------------------------------------------------------------------------------------------

            Console.WriteLine("\n========== Import finished @ {0} ============", DateTime.Now);
            return root;
        }

        #region Print Some Stuff

        private static void PrintSomeStuff(katum root)
        {
            katum tables = root.find("Table");
            katum columns = root.find("Column");

            Console.WriteLine("--- Print Stats ---");
            PrintStats(tables, columns);

            Console.WriteLine("--- Print Table Stats ---");
            tables.doI(PrintTableStats);

            Console.WriteLine("--- Print Table Columns ---");
            tables.doI(t => PrintTableColumns(columns, t));

            Console.WriteLine("--- Print small tables ---");
            PrintTableRows(columns, tables & "Category");
            PrintTableRows(columns, tables & "Region");
            PrintTableRows(columns, tables & "Shipper");
            PrintTableRows(columns, tables & "Employee");
        }

        private static void PrintStats(katum tables, katum columns)
        {
            Console.WriteLine("Tables");
            tables.doI(Console.WriteLine);

            Console.WriteLine("Columns");
            columns.doI(Console.WriteLine);
        }

        private static void PrintTableStats(katum table)
        {
            Console.WriteLine("{0} has {1} rows", table, table.countI);
        }

        private static void PrintTableColumns(katum columns, katum table)
        {
            columns[table].doI(Console.WriteLine);
        }

        private static void PrintTableRows(katum columns, katum table)
        {
            Console.WriteLine();
            katum tableColumns = columns[table];
            foreach (katum i in table.I)
            {
                Console.WriteLine();
                foreach (katum column in tableColumns.I)
                    if(column[i] != null)
                        Console.Write("[{0}] ", column[i].O);
            }
        }

        #endregion
    }
}
